from algorithms import compute_realization_tagged, compute_realization_canonical, compute_realization_geometric

# a triplet of size-4 permutations (permutations of {0, ..., 3})
perm_triplet = [[0,2,3,1], [1,3,0,2], [2,3,1,0]]
# search for a canonical realization (i.e. in this order on lx, ly and lz resp.)
# the output is [x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4] (see the paper)
print(compute_realization_canonical(perm_triplet))

print('-'*20)

# a triplet of positions of [zero, one] (i.e. the number of elements smaller than zero and one)
# a matching realization
tag_triplet = [[0, 2], [1, 2], [1, 3]]
print(tag_triplet, compute_realization_tagged(perm_triplet, tag_triplet))

# another triplet with no realization
tag_triplet = [[0, 0], [0, 3], [2, 4]]
print(tag_triplet, compute_realization_tagged(perm_triplet, tag_triplet))

print('-'*20)

perm_triplet = [[3, 1, 2, 0], [1, 2, 0, 3], [2, 0, 3, 1]]
# this perm triplet is not canonically realizable
print(compute_realization_canonical(perm_triplet))
# but, as all triplet of size-4 permutations, it is geometrically realizable
print(compute_realization_geometric(perm_triplet))


perm_triplet = [[1, 0], [1, 0], [0, 1]]
print(compute_realization_canonical(perm_triplet))
# but, as all triplet of size-4 permutations, it is geometrically realizable
print(compute_realization_geometric(perm_triplet))

print('-'*20)
print("Computer proof of Theorem 2")
print('-'*20)
#$(abcd, dcba, badc)$, $(dcba, abcd, badc)$, $(abcd, dcba,cdab)$, $(dcba, abcd, cdab)$
perm_triplet = [[0,1,2,3], [3,2,1,0], [1,0,3,2]] # (abcd, dcba, badc)
print("(abcd, dcba, badc)", compute_realization_canonical(perm_triplet))

perm_triplet = [[3,2,1,0], [0,1,2,3], [1,0,3,2]] # (dcba, abcd, badc)
print("(dcba, abcd, badc)", compute_realization_canonical(perm_triplet))

perm_triplet = [[0,1,2,3], [3,2,1,0], [2,3,0,1]] # (abcd, dcba, cdab)
print("(abcd, dcba, cdab)", compute_realization_canonical(perm_triplet))

perm_triplet = [[3,2,1,0], [0,1,2,3], [2,3,0,1]] # (dcba, abcd, cdab)
print("(dcba, abcd, cdab)", compute_realization_canonical(perm_triplet))