from algorithms import compute_realization_tagged, inverse_perm, mul_perm, contains_forbidden_pattern
from itertools import permutations, combinations, combinations_with_replacement, product


def all_normalized_forbidden(n, begin=None, end=None):
    """Loop over all canonically normalized tagged triplet of a given size n and lists the minimal
    forbidden patterns. According to our conjecture, this should be empty. If 'begin' and 'end' are specified,
    the loop is over the tagged triplets [id,sigma,tau] where sigma is from the permutation number 'begin' and
    the permutation number 'end' (included)"""
    LP = list(permutations(range(n)))
    LZO = list(combinations_with_replacement(range(n+1),2))
    px = list(range(n))
    nbp = 0
    R = []
    for py, pz in product(LP, LP):
        py, pz = list(py), list(pz)
        if pz == px:
            if nbp > 0 and (begin is None or nbp >= begin):
                print("    done ("+str(nbp)+")")
            if end is not None and end == nbp:
                return
            nbp += 1
            if begin is  None or nbp >= begin:
                print("starting ("+str(nbp)+")", py)
        if begin is not None and nbp < begin:
            continue
        for zox, zoy, zoz in product(LZO, LZO, LZO):
            zox, zoy, zoz = list(zox), list(zoy), list(zoz)
            perm_triplet = [px, py, pz]
            tag_triplet = [zox, zoy, zoz]
            if is_normalized_tagged_triplet(perm_triplet, tag_triplet):
                if contains_forbidden_pattern(perm_triplet,tag_triplet):
                    continue
                if not compute_realization_tagged(perm_triplet, tag_triplet):
                    R.append([perm_triplet, tag_triplet])
    return R

def all_normalized_geom(n):
    """Computes all normalized geometric triplets of size n"""
    L = [list(x) for x in permutations(range(n)) if x[0]<x[-1]]
    A = []
    identity = list(range(n))
    for i,j in combinations(range(1,len(L)),2):
        triplet = [identity,L[i],L[j]]
        if is_normalized_geom(triplet):
            A.append(triplet)
    return A

def normalize_geom(triplet):
    px,py,pz = triplet
    n = len(px)
    identity = list(range(n))
    L = []
    ipx = inverse_perm(px)
    ipy = inverse_perm(py)
    ipz = inverse_perm(pz)
    rx = px[::-1]
    ry = py[::-1]
    rz = pz[::-1]
    irx = inverse_perm(rx)
    iry = inverse_perm(ry)
    irz = inverse_perm(rz)
    # candidate start with px relabelled
    C = [identity]
    y = min(mul_perm(ipx,py), mul_perm(ipx,ry))
    z = min(mul_perm(ipx,pz), mul_perm(ipx,rz))
    C.append(min(y,z))
    C.append(max(y,z))
    L.append(C)
    # candidate start with rx relabelled
    C = [identity]
    y = min(mul_perm(irx,py), mul_perm(irx,ry))
    z = min(mul_perm(irx,pz), mul_perm(irx,rz))
    C.append(min(y,z))
    C.append(max(y,z))
    L.append(C)
    # candidate start with py relabelled
    C = [identity]
    x = min(mul_perm(ipy,px), mul_perm(ipy,rx))
    z = min(mul_perm(ipy,pz), mul_perm(ipy,rz))
    C.append(min(x,z))
    C.append(max(x,z))
    L.append(C)
    # candidate start with ry relabelled
    C = [identity]
    x = min(mul_perm(iry,px), mul_perm(iry,rx))
    z = min(mul_perm(iry,pz), mul_perm(iry,rz))
    C.append(min(x,z))
    C.append(max(x,z))
    L.append(C)
    # candidate start with pz relabelled
    C = [identity]
    x = min(mul_perm(ipz,px), mul_perm(ipz,rx))
    y = min(mul_perm(ipz,py), mul_perm(ipz,ry))
    C.append(min(x,y))
    C.append(max(x,y))
    L.append(C)
    # candidate start with rz relabelled
    C = [identity]
    x = min(mul_perm(irz,px), mul_perm(irz,rx))
    y = min(mul_perm(irz,py), mul_perm(irz,ry))
    C.append(min(x,y))
    C.append(max(x,y))
    L.append(C)
    return min(L)


def is_normalized_geom(triplet):
    return triplet == normalize_geom(triplet)


def normalize_tagged_triplet(perm_triplet,zo_triplet):
    inv_triplet = [inverse_perm(p) for p in perm_triplet]
    px, py, pz = perm_triplet
    ix, iy, iz = inv_triplet
    zox, zoy, zoz = zo_triplet
    id = list(range(len(px)))
    R0 = [[id, mul_perm(ix,py),mul_perm(ix,pz)],[zox,zoy,zoz]]
    R1 = [[id, mul_perm(iy,pz),mul_perm(iy,px)],[zoy,zoz,zox]]
    R2 = [[id, mul_perm(iz,px),mul_perm(iz,py)],[zoz,zox,zoy]]
    return min(R0,R1,R2)


def is_normalized_tagged_triplet(perm_triplet,zo_triplet):
    p3, zo3 = normalize_tagged_triplet(perm_triplet,zo_triplet)
    return p3 == perm_triplet and zo3 == zo_triplet
